@@ -1,11 +1,10 @@
-#!/usr/bin/env perl
 package Class::Load;
 use strict;
 use warnings;
 use base 'Exporter';
 use File::Spec;
 
-our $VERSION = '0.03';
+our $VERSION = '0.05';
 
 our @EXPORT_OK = qw/load_class try_load_class is_class_loaded/;
 our %EXPORT_TAGS = (
@@ -14,6 +13,12 @@ our %EXPORT_TAGS = (
 
 our $ERROR;
 
+BEGIN {
+    *IS_RUNNING_ON_5_10 = ($] < 5.009_005)
+        ? sub () { 0 }
+        : sub () { 1 };
+}
+
 sub load_class {
     my $class = shift;
 
@@ -47,30 +52,63 @@ sub try_load_class {
     return 0;
 }
 
+sub _is_valid_class_name {
+    my $class = shift;
+
+    return 0 if ref($class);
+    return 0 unless defined($class);
+    return 0 unless length($class);
+
+    return 1 if $class =~ /^\w+(?:::\w+)*$/;
+
+    return 0;
+}
+
 sub is_class_loaded {
     my $class = shift;
 
-    # is the module's file in %INC?
-    my $file = (join '/', split '::', $class) . '.pm';
-    return 1 if $INC{$file};
+    return 0 unless _is_valid_class_name($class);
 
-    # any interesting symbols in this module's symbol table?
-    my $table = do {
-        no strict 'refs';
-        \%{ $class . '::' };
-    };
+    # walk the symbol table tree to avoid autovififying
+    # \*{${main::}{"Foo::"}} == \*main::Foo::
 
-    # ..such as @ISA?
-    return 1 if exists $table->{ISA};
-
-    # ..such as $VERSION?
-    return 1 if exists $table->{VERSION};
-
-    # ..or a method?
-    for my $glob (values %$table) {
-        return 1 if *{$glob}{CODE};
+    my $pack = \*::;
+    foreach my $part (split('::', $class)) {
+        return 0 unless exists ${$$pack}{"${part}::"};
+        $pack = \*{${$$pack}{"${part}::"}};
     }
 
+    # We used to check in the package stash, but it turns out that
+    # *{${$$package}{VERSION}{SCALAR}} can end up pointing to a
+    # reference to undef. It looks
+
+    my $version = do {
+        no strict 'refs';
+        ${$class . '::VERSION'};
+    };
+
+    return 1 if ! ref $version && defined $version;
+    # Sometimes $VERSION ends up as a reference to undef (weird)
+    return 1 if ref $version && reftype $version eq 'SCALAR' && defined ${$version};
+
+    return 1 if exists ${$$pack}{ISA}
+             && defined *{${$$pack}{ISA}}{ARRAY};
+
+    # check for any method
+    foreach ( keys %{$$pack} ) {
+        next if substr($_, -2, 2) eq '::';
+
+        my $glob = ${$$pack}{$_} || next;
+
+        # constant subs
+        if ( IS_RUNNING_ON_5_10 ) {
+            return 1 if ref $glob eq 'SCALAR';
+        }
+
+        return 1 if defined *{$glob}{CODE};
+    }
+
+    # fail
     return 0;
 }
 
@@ -96,14 +134,17 @@ Class::Load - a working (require "Class::Name") and more
 =head1 DESCRIPTION
 
 C<require EXPR> only accepts C<Class/Name.pm> style module names, not
-C<Class::Name>. How frustrating!
+C<Class::Name>. How frustrating! For that, we provide
+C<load_class 'Class::Name'>.
 
 It's often useful to test whether a module can be loaded, instead of throwing
-an error when it's not available.
+an error when it's not available. For that, we provide
+C<try_load_class 'Class::Name'>.
 
 Finally, sometimes we need to know whether a particular class has been loaded.
 Asking C<%INC> is an option, but that will miss inner packages and any class
-for which the filename does not correspond to the package name.
+for which the filename does not correspond to the package name. For that, we
+provide C<is_class_loaded 'Class::Name'>.
 
 =head1 FUNCTIONS
 
@@ -127,8 +168,8 @@ packages which C<require> does not check.
 =head2 is_class_loaded Class::Name -> 0|1
 
 This uses a number of heuristics to determine if the class C<Class::Name> is
-loaded. We check first whether it has an entry in C<%INC>, then check its
-symbol table for interesting values.
+loaded. There heuristics were taken from L<Class::MOP>'s old pure-perl
+implementation.
 
 =head1 SEE ALSO
 
@@ -147,7 +188,7 @@ C<is_class_loaded>.
 =item L<Moose>, L<Jifty>, L<Prophet>, etc
 
 This module was designed to be used anywhere you have
-C<if (eval "require $module"; 1)>.
+C<if (eval "require $module"; 1)>, which occurs in many large projects.
 
 =back
 
@@ -155,6 +196,8 @@ C<if (eval "require $module"; 1)>.
 
 Shawn M Moore, C<< <sartak at bestpractical.com> >>
 
+The implementation of C<is_class_loaded> has been taken from L<Class::MOP>.
+
 =head1 BUGS
 
 Please report any bugs or feature requests to
@@ -163,7 +206,7 @@ L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Class-Load>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 Best Practical Solutions.
+Copyright 2008-2009 Best Practical Solutions.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
